/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



#define VER             "0.1"
#define BUFFSZ          4096
#define NICK            "usr_nick="
#define PASS            "usr_pass="



#ifndef WIN32
    #define strnicmp strncasecmp
#endif

#ifndef u_char
    #define u_char  unsigned char
#endif



void gsa_pwd(u_char *pwd);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u_char  *buff,
            *ptr,
            *limit;


    setbuf(stdout, NULL);

    fputs("\n"
        "Gamespy Arcade password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <settings.cfg>\n"
            "\n"
            "settings.cfg is located in the directory Profiles\\ID where ID is the profile number\n"
            "\n", argv[0]);
        fputs("\n    Press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    printf("\nOpening file %s\n\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err();

    buff = malloc(BUFFSZ + 1);
    if(!buff) std_err();

    while(1) {
        if(!fgets(buff, BUFFSZ, fd)) break;

        if(!strnicmp(buff, NICK, sizeof(NICK) - 1)) {
            printf("Nickname:   %s", buff + sizeof(NICK) - 1);

        } else if(!strnicmp(buff, PASS, sizeof(PASS) - 1)) {
            ptr = buff + sizeof(PASS) - 1;
            if(*ptr == 0xab) {  // new method, encoded
                ptr++;
                limit = strchr(ptr, 0xbb);
                if(limit) *limit = 0;
                gsa_pwd(ptr);
            } else {            // old method
                for(limit = ptr; *limit > 0xd; limit++);
                *limit = 0;
            }
            printf("Password:   %s\n", ptr);
        }
    }

    fclose(fd);
    fputs("\n\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void gsa_pwd(u_char *pwd) {
    u_char  *out;
    int     len,
            tmp;

    out = pwd;
    len = strlen(pwd) >> 1;
    while(len--) {
        tmp = *pwd++;
        tmp = *pwd++ - tmp;
        if(tmp < 0) tmp += 0x5f;
        *out++ = tmp + 0x20;
    }
    *out = 0;
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


